Search.setIndex({objtypes:{"0":"py:module","1":"py:class","2":"py:method","3":"py:exception"},terms:{current:0,geoip2error:0,contact:0,precis:0,most:0,page:0,problem:0,similar:0,"import":0,around:0,variabl:0,accept:0,usag:0,two:0,runtimeerror:0,thi:0,confid:0,"class":0,continent_cod:0,still:0,is_transparent_proxi:0,like:0,possibl:0,geoip:0,dictionari:0,geoip2webserviceerror:0,citi:0,dmaregion:0,com:0,need:0,webservic:0,"return":0,googl:0,www:0,deliv:0,empti:0,is_anonymous_proxi:0,edit:0,queri:0,which:0,each:0,lookup:0,contin:0,russian:0,further:0,gnu:0,run:0,becaus:0,web:0,indic:0,some:0,locat:0,onc:0,"4xx":0,raw_respons:0,add:0,geoname_id:0,field:0,paramet:0,vari:0,geoip2httperror:0,"throw":0,geonam:0,recordscountri:0,dev:0,postal_cod:0,behind:0,alpha:0,ipv4:0,valid:0,keyword:0,point:0,identifi:0,subclass:0,argument:0,militari:0,want:0,distribut:0,other:0,note:0,statu:0,local:0,japanes:0,iana:0,accent:0,govern:0,gener:0,isp:0,incorrect:0,besid:0,avail:0,thing:0,occur:0,print:0,simplifi:0,thei:0,"long":0,should:0,request:0,also:0,iso_3166_2:0,python:0,autonom:0,link:0,issu:0,longitud:0,onto:0,model:0,associ:0,mistak:0,metro:0,about:0,explicitli:0,thrown:0,later:0,"5xx":0,when:0,modul:0,databas:0,part:0,"while":0,tracker:0,ssl:0,librari:0,"public":0,kei:0,north:0,product:0,place:0,radiu:0,error:0,abcdef123456:0,piec:0,chines:0,address:0,popul:0,where:0,cityisporg:0,code:0,school:0,detail:0,english:0,second:0,busi:0,geoip2:0,api:0,assign:0,becom:0,download:0,organ:0,free:0,cellular:0,name:0,appendix:0,attribut:0,omni:0,latitud:0,three:0,region:0,iso:0,iso_3166_1_alpha_2:0,iso_3166_1_alpha_3:0,releas:0,org:0,mani:0,level:0,everi:0,string:0,search:0,user_typ:0,correspond:0,spell:0,handl:0,can:0,host:0,found:0,languag:0,order:0,fail:0,instal:0,differ:0,even:0,"case":0,incorpor:0,"default":0,factor:0,call:0,accuracy_radiu:0,record:0,first:0,"true":0,itself:0,creat:0,word:0,invalid:0,content_delivery_network:0,work:0,my_license_kei:0,valu:0,addit:0,"final":0,adword:0,unanticip:0,premium:0,routabl:0,sourc:0,support:0,submit:0,site:0,"float":0,been:0,iso_3166:0,data:0,entir:0,ani:0,endpoint:0,predic:0,make:0,search_engine_spid:0,arg:0,colleg:0,more:0,develop:0,registered_countri:0,least:0,charact:0,regist:0,map:0,trait:0,tabl:0,futur:0,ip_address:0,router:0,wikipedia:0,follow:0,number:0,all:0,featur:0,residenti:0,incorrectli:0,paid:0,list:0,license_kei:0,time:0,see:0,lgplv2:0,anonym:0,ascii:0,zone:0,succe:0,internet:0,doe:0,explicit:0,doc:0,have:0,http_statu:0,correct:0,postal:0,placerecord:0,http:0,hostnam:0,new_york:0,uniqu:0,between:0,inform:0,foo:0,own:0,repres:0,autonomous_system_numb:0,provid:0,believ:0,mai:0,etc:0,view:0,"new":0,world:0,onli:0,made:0,specifi:0,latter:0,descript:0,autonomous_system_organ:0,metro_cod:0,constructor:0,result:0,block:0,given:0,autonomous_system_:0,you:0,cafe:0,report:0,set:0,from:0,sort:0,automat:0,basic:0,servic:0,pass:0,user:0,turn:0,check:0,document:0,base:0,specif:0,older:0,help:0,sure:0,pleas:0,alwai:0,someth:0,object:0,travel:0,present:0,bodi:0,extend:0,domain:0,find:0,end:0,proxi:0,time_zon:0,both:0,against:0,except:0,properti:0,nat:0,city_isp_org:0,exampl:0,mean:0,extern:0,multipl:0,index:0,messag:0,geograph:0,perform:0,look:0,none:0,type:0,method:0,postal_confid:0,maxmind:0,copyright:0,user_id:0,oceania:0,version:0,custom:0,faq:0,america:0,contain:0,unpopul:0,github:0,transport:0,portion:0,transpar:0,what:0,yet:0,wiki:0,licens:0,integ:0,iso_3166_alpha_2:0,system:0,move:0,ipv6:0,includ:0,client:0,lesser:0,inc:0,requir:0,hass:0,anonproxi:0,offer:0,your:0,altern:0,same:0,uri:0,particular:0,integr:0,signific:0,pai:0,kilomet:0,countri:0},objects:{"":{geoip2:[0,0,1,""]},geoip2:{webservices:[0,0,1,""],errors:[0,0,1,""],models:[0,0,1,""],records:[0,0,1,""]},"geoip2.errors":{GeoIP2Error:[0,3,1,""],GeoIP2HTTPError:[0,3,1,""],GeoIP2WebServiceError:[0,3,1,""]},"geoip2.models":{Omni:[0,1,1,""],CityISPOrg:[0,1,1,""],Country:[0,1,1,""],City:[0,1,1,""]},"geoip2.records":{Traits:[0,1,1,""],PlaceRecord:[0,1,1,""],Record:[0,1,1,""],Location:[0,1,1,""],City:[0,1,1,""],Continent:[0,1,1,""],Region:[0,1,1,""],Country:[0,1,1,""]},"geoip2.webservices.Client":{omni:[0,2,1,""],city_isp_org:[0,2,1,""],country:[0,2,1,""],city:[0,2,1,""]},"geoip2.webservices":{Client:[0,1,1,""]}},filenames:["index"],objnames:{"0":["py","module","Python module"],"1":["py","class","Python class"],"2":["py","method","Python method"],"3":["py","exception","Python exception"]},titles:["MaxMind GeoIP2 Python API"]})